                .LF     et.lst
;------------------------------------------------------------------------
;
;  et.asm
;
;  A first real program, an egg timer
;
;  Author : San Bergmans
;  Source : www.sbprojects.com
;  Date   : 9-12-2005
;
;------------------------------------------------------------------------
;
;  This program functions as a sort of egg timer. You can set the time
;  (from 1 second up to 99 minutes and 59 seconds!) and then start the
;  timer. The timer then counts backwards to 0 at which point it will
;  sound the alarm.
;
;------------------------------------------------------------------------

                .CR     Z80             Yeah! We're programming the Z80!
                .OR     01800H          RAM starts here, so are we
                .TF     et.hex,HEX      Create a plain hex dump file

;------------------------------------------------------------------------
;  Constants
;------------------------------------------------------------------------

SEC_RELOAD      .EQ     99              Fraction of second reload value

GO              .EQ     012H            GO key  (SCAN)
DEL             .EQ     017H            DEL key (SCAN)

STEP1           .EQ     010H            STEP key  (SCAN1)

SA              .EQ     00001000B       Segment A
SB              .EQ     00010000B       Segment B
SC              .EQ     00100000B       Segment C
SD              .EQ     10000000B       Segment D
SE              .EQ     00000001B       Segment E
SF              .EQ     00000100B       Segment F
SG              .EQ     00000010B       Segment G
DP              .EQ     01000000B       Decimal point

;------------------------------------------------------------------------
;  Monitor routines
;------------------------------------------------------------------------

SCAN            .EQ     005FEH          Scan display until a key pressed
SCAN1           .EQ     00624H          Scan display once
HEX7            .EQ     00689H          Convert hex digit to 7-segement
HEX7SG          .EQ     00678H          Convert 2 hex digits to 7-segment
TONE1K          .EQ     005DEH          Sound a 1kHz tone during HL

;------------------------------------------------------------------------
;  START
;  The program starts here
;------------------------------------------------------------------------

ET              LD      HL,(MINUTES_DEF) Set the default starting time
                LD      (MINUTES),HL      (2 bytes at a time!)
                LD      A,SEC_RELOAD    Set seconds fraction counter
                LD      (FRACTION),A

IDLE            CALL    SHOW_TIME       Convert BCD time to 7-segments

.LOOP           LD      IX,DSP_BFFR     Scan display until key
                CALL    SCAN
                CP      A,GO            Is it the GO key?
                JR      Z,RUN           Yes! Start the count down
                CP      A,10            Is it a digit key? ( A<10 )
                JR      C,ENTER         Yes! Start entering a new time
                CP      A,DEL           Is it the DEL key?
                JR      Z,ET            Yes! Reload default value
                JR      .LOOP

;------------------------------------------------------------------------
;  RUN
;  The countdown is running
;------------------------------------------------------------------------

RUN             CALL    SHOW_TIME       Convert BCD time to 7-segment

.LOOP           CALL    SCAN1           Scan loop once
                JR      C,.CONT         No key was pressed
                CP      A,STEP1         Request to stop?
                JR      Z,IDLE          Yes! Stop count down

.CONT           LD      HL,FRACTION     Decrement the fraction part
                DEC     (HL)
                JR      NZ,.LOOP        Fraction not 0 yet!
                LD      (HL),SEC_RELOAD Reset the fraction part

                LD      HL,SECONDS      Decrement the seconds part
                LD      A,(HL)
                SUB     1               Can't use DEC because of DAA
                DAA                     Adjust to packed BCD number
                LD      (HL),A
                JR      NC,.CHECK       No underflow into minutes!
                LD      (HL),059H       Reset the seconds counter (BCD!)

                LD      HL,MINUTES      Decrement the minutes part
                LD      A,(HL)
                SUB     1               Can't use DEC because of DAA
                DAA                     Adjust to packed BCD number
                LD      (HL),A
                JR      RUN             Never underflows!!!!

.CHECK          LD      HL,MINUTES      See if time is now 0:00
                OR      A,(HL)           (Z=1 if both A and (HL) are 0)
                JR      NZ,RUN          It's not 0.00 yet!

;------------------------------------------------------------------------
;  STOP
;  Timed out, show -STOP- and beep
;------------------------------------------------------------------------

STOP            LD      IX,.DSP_BFFR    Show -STOP- on the display
.LOOP
                LD      HL,500          Sound the alarm for half a
                CALL    TONE1K           second

                LD      B,50            Then display -STOP- for half a
.DISPLAY        CALL    SCAN1            second
                JR      NC,.EXIT        Until any key is pressed!
                DJNZ    .DISPLAY        Delay over when B becomes 0!
                JR      .LOOP           Repeat alarm/flash sequence

.EXIT           JR      ET              Restart program

.DSP_BFFR       .DA     #SG                     -
                .DA     #SA+SB+SE+SF+SG         P
                .DA     #SA+SB+SC+SD+SE+SF      O
                .DA     #SD+SE+SF+SG            T
                .DA     #SA+SC+SD+SF+SG         S
                .DA     #SG                     -

;------------------------------------------------------------------------
;  ENTER
;  Enter a new time here
;------------------------------------------------------------------------

ENTER           LD      B,A
                RLCA                    Move first digit to MSD of the
                RLCA                     minutes
                RLCA
                RLCA
                LD      E,A             And save edit time in DE
                LD      D,0
                LD      HL,1000000010000000B  Show digit and 3 dashes
                LD      (DSP_BFFR),HL
                LD      A,B             Get digit back and display it
                CALL    HEX7
                LD      H,A              along with a dash and a dot
                LD      L,10000000B+DP
                LD      (DSP_BFFR+2),HL

.MIN            CALL    SCAN            Show this time until key
                CP      A,DEL           Was DEL key pressed?
                JR      Z,.IDLE         Yes! Forget everything
                CP      A,10            Is it a digit key?
                JR      NC,.MIN         Nope!
                LD      B,A
                OR      A,E             Combine this digit with previous
                LD      E,A              to form a packed BCD
                LD      A,B             Get digit back and display it
                CALL    HEX7
                OR      A,DP             don't forget the DP to the right
                LD      (DSP_BFFR+2),A    of this digit

.SEC2           CALL    SCAN            Show this time until key
                CP      A,DEL           Was DEL key pressed?
                JR      Z,.IDLE         Yes! Forget everything
                CP      A,6             Is it a legal digit key? (0..5)
                JR      NC,.SEC2        No! ( A>5 )
                LD      B,A
                RLCA                    Move first digit to MSD
                RLCA
                RLCA
                RLCA
                LD      D,A              and save it (in DE)
                LD      A,B             Get digit back and display it
                CALL    HEX7
                LD      (DSP_BFFR+1),A

.SEC            CALL    SCAN            Show this time until key
                CP      A,DEL           Was DEL key pressed?
                JR      Z,.IDLE         Yes! Forget everything
                CP      A,10            Was it a digit key?
                JR      NC,.SEC         No!
                OR      A,D             Combine digit with previous one

                LD      H,A             Set new time
                LD      L,E
                OR      A,L             Is total time 0?
                JR      Z,.SEC          Yes! Don't allow this!
                LD      (MINUTES),HL    Save 2 packed BCD bytes
                LD      (MINUTES_DEF),HL Make it also the default now!
                LD      A,SEC_RELOAD    Set seconds fraction counter to
                LD      (FRACTION),A     one whole second

.IDLE           JP      IDLE            Go back to stop mode

;------------------------------------------------------------------------
;  SHOW_TIME
;  Convert current time to 7-segment before we can display it
;------------------------------------------------------------------------

SHOW_TIME       LD      HL,DSP_BFFR     Convert seconds to 7-segments
                LD      A,(SECONDS)
                CALL    HEX7SG
                LD      A,(MINUTES)     Convert minutes to 7-segments
                LD      B,A
                CALL    HEX7            Can't do 2 at a time now because
                OR      A,01000000B      of the DP between MIN & SEC
                LD      (HL),A           and because of leading 0 blank
                INC     HL
                LD      A,B
                RRCA                    Convert MSD of minutes too
                RRCA
                RRCA
                RRCA
                AND     A,00001111B
                JR      Z,.LEAD0        It's a leading 0, don't display!
                CALL    HEX7
                LD      (HL),A
                RET

.LEAD0          XOR     A,A             Blank the display of minutes MSD
                LD      (HL),A
                RET

;------------------------------------------------------------------------
;  VARIABLES
;------------------------------------------------------------------------

MINUTES_DEF     .DA     #003H           Default minutes
SECONDS_DEF     .DA     #000H           Default seconds
MINUTES         .DA     #000H           BCD value of minute down counter
SECONDS         .DA     #000H           BCD value of seconds down counter
FRACTION        .DA     #000H           Fraction of second counter

;------------------------------------------------------------------------

DSP_BFFR        .DA     #0              Display buffer (scanned from
                .DA     #0               right to left)
                .DA     #0
                .DA     #0
                .DA     #SD+SE+SF+SG     't'
                .DA     #SA+SD+SE+SF+SG  'E'

;------------------------------------------------------------------------
                .LI     OFF
